import os
import sys
import platform
import re
import htmlentitydefs
import urllib2
import math
import locale
	
def seconds_to_hms(seconds):
	hours = seconds / 3600
	seconds -= 3600 * hours
	minutes = seconds / 60
	seconds -= 60 * minutes
	return "%02d:%02d:%02d.000" % (hours, minutes, seconds)

def hex2char(m):
	return chr(int(m.group(1), 16))

def urldecode(url):
	rex = re.compile('%([0-9a-hA-H][0-9a-hA-H])', re.M)
	return rex.sub(hex2char,url)
	
def stripslashes(s):
	r = re.sub(r"\\(n|r)", "\n", s)
	r = re.sub(r"\\", "", r)
	return r

def smart_size(bytes):
	if bytes is None:
		return 'N/A'
	if type(bytes) is str:
		bytes = float(bytes)
	if bytes == 0.0:
		exponent = 0
	else:
		exponent = long(math.log(bytes, 1024.0))
	suffix = 'bkMGTPEZY'[exponent]
	converted = float(bytes) / float(1024**exponent)
	return '%.2f%s' % (converted, suffix)
	
def htmlentity_transform(matchobj):
	"""Transforms an HTML entity to a Unicode character.
	
	This function receives a match object and is intended to be used with
	the re.sub() function.
	"""
	entity = matchobj.group(1)

	# Known non-numeric HTML entity
	if entity in htmlentitydefs.name2codepoint:
		return unichr(htmlentitydefs.name2codepoint[entity])

	# Unicode character
	mobj = re.match(ur'(?u)#(x?\d+)', entity)
	if mobj is not None:
		numstr = mobj.group(1)
		if numstr.startswith(u'x'):
			base = 16
			numstr = u'0%s' % numstr
		else:
			base = 10
		return unichr(long(numstr, base))

	# Unknown entity in name, return its literal representation
	return (u'&%s;' % entity)


def unhtmlentities(utitle):
	utitle = re.sub(ur'(?u)&(.+?);', htmlentity_transform, utitle)
	return utitle
	
def normalize_filename(utitle):
	utitle = re.sub(ur'(?u)&(.+?);', htmlentity_transform, utitle)
	utitle.replace(unicode(os.sep), u'%')
	utitle = re.sub(ur'[/<>:"\|\?\*]', u'#', utitle)
	return utitle
	
def parse_bytes(bytestr):
	"""Parse a string indicating a byte quantity into a long integer."""
	matchobj = re.match(r'(?i)^(\d+(?:\.\d+)?)([kMGTPEZY]?)$', bytestr)
	if matchobj is None:
		return None
	number = float(matchobj.group(1))
	multiplier = 1024.0 ** 'bkmgtpezy'.index(matchobj.group(2).lower())
	return long(round(number * multiplier))
	


def localedir():
	if platform.system() == "Windows":
		DIR = "locale"
		os.environ['LANG'] = locale.getdefaultlocale()[0]
	else:
		if os.path.exists('./locale'):
			DIR = os.path.join(os.path.dirname(sys.argv[0]),"locale")
		else:
			DIR = "/usr/share/locale"
	return DIR
