#!/usr/bin/python
#Licence: GPLv3.0
#Copyright: Krasimir S. Stefanov <lokiisyourmaster@gmail.com>

import os
import sys
import platform
import re
import urllib
import shlex
import asubprocess
import locale
import gettext
import mimetypes
from urlparse import urlparse
from optparse import OptionParser
try:
	from urlparse import parse_qs
except ImportError:
	from cgi import parse_qs
from functions import *

# localization
APP = "viget"
PLUGINS_VER = "0.2.4"
DIR = localedir()
gettext.bindtextdomain(APP, DIR)
gettext.textdomain(APP)
gettext.install(APP, localedir=DIR)
_ = gettext.gettext
# /localization

COOKIE_FILE = os.path.join(os.getenv('HOME') or os.getenv('USERPROFILE'),".viget_cookies")

	
class WebVideoDownloader(object):
	re = None
	website = None
	url = None
	source = None
	video_urls = []
	returndata = None
	subs = None
	headers = []
	login_data = None
	login_url = None
	login_required = False
	user_agent = 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.13) Gecko/20101206 USU/6.0 Firefox/3.6.13'	
	title = None
	detected = None
	
	def __init__(self, url = None):
		self.url = url.strip()
		
	def detect(self):
		if self.url:
			if type(self.re) is list:
				for r in self.re:
					if re.search(r, self.url) is not None:
						self.detected = True
						return True
			elif type(self.re) is str:
				self.detected = re.search(self.re, self.url)
				return (self.detected is not None)			
			else:
				return False

		else:
			self.detected = None
			return False
		
	def get_video_url(self):
		return self.video_urls
		
	def get_subtitles(self):
		return
		
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<title>(.*?)</title>', self.source, re.DOTALL)
		if m:
			self.title = m.group(1).decode('utf-8').replace('\n','').replace('\r','')
		else:
			self.title = None
		return self.title
		
	def get_output_name(self):
		return
	
	def get_source(self):
		if self.url:
			self.source = self.http_request(self.url)
			return self.source
		else:
			return None
	
	def login(self, headers = []):
		return self.http_request(self.login_url, self.login_data, headers)
		
	def http_request(self, url, post_data = None, headers = [], raw_post = False):
		if not os.path.exists(COOKIE_FILE):
			open(COOKIE_FILE, "a+").close()
			
		if not post_data:
			args = 'curl --cookie-jar "%s" --user-agent "%s" --url "%s"' % (COOKIE_FILE, self.user_agent, url)
		else:
			
			if raw_post:
				args = 'curl --data-binary "%s" --cookie-jar "%s" --user-agent "%s" --url "%s"' % (post_data, COOKIE_FILE, self.user_agent, url)
			else:
				params = urllib.urlencode(post_data)
				args = 'curl --data "%s" --cookie-jar "%s" --user-agent "%s" --url "%s"' % (params, COOKIE_FILE, self.user_agent, url)

		if len(self.headers):
			for header in self.headers:
				args = args + ' --header "%s"' % header

		if len(headers):
			for header in headers:
				args = args + ' --header "%s"' % header
		
		if platform.system() != "Windows":
			args = args +' -4'
		args = args + ' --connect-timeout 10 --max-time 10 --insecure --location'

		argv =  shlex.split(args)
		info = None
		if platform.system() == "Windows":
			info = asubprocess.STARTUPINFO()
			info.dwFlags |= asubprocess.STARTF_USESHOWWINDOW
			info.wShowWindow = asubprocess.SW_HIDE

		process = asubprocess.Popen(argv, stderr=asubprocess.PIPE, stdout=asubprocess.PIPE, startupinfo=info)
		return process.stdout.read()
				
class vbox7(WebVideoDownloader):
	re = '(www\.)?vbox7.com/play:([a-z0-9]{8})'
	website = 'http://vbox7.com'
	login_url = 'http://www.vbox7.com/login/in.do'
	login_data = {
		'vboxusr': ''
		, 'vboxpwd': ''
		, 'rememberme': '1'
	}
	login_required = True

	def get_video_url(self):
		#TODO: confirm age 
		#if self.login_data['vboxusr'] and self.login_data['vboxpwd']:
		#	print self.login()
		#	data = {
		#		'confirm':'1'
		#	}
		#	print self.http_request('http://www.vbox7.com/etc/parent.do', data)
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		m = re.search(self.re, self.url)
		if m:
			self.videoID = m.group(2)
			params = urllib.urlencode({
				'key': m.group(2)
			})
			url = "http://www.vbox7.com/etc/ext.do?%s" % params
			f = urllib.urlopen(url)
			self.returndata = f.read()
			m = re.search('&flv_addr=(.*?)&', self.returndata)
			if m:
				videoUrl = m.group(1)
				if re.search('^(http:\/\/|https:\/\/)', videoUrl) == None:
					videoUrl = 'http://' + videoUrl
				
				videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
				self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title)+videoExt]]
			else:
				self.video_urls = []
			return self.video_urls
			
	def get_subtitles(self):
		if not self.video_urls:
			if not self.get_video_url():
				return None

		m = re.search('http://i\d+\.vbox7\.com/subs/' + self.videoID[:2] + '/' + self.videoID + '_.*?\.js\?.*?', self.source)
		if m:
			subsJSON = self.http_request(m.group(0)).encode('iso-8859-1').replace('\\"','"')
			s = re.compile('{"s":"(.*?)","t"\:(\d+),"f"\:(\d+)}')
			items = s.findall(subsJSON)
			row = 0
			self.subs = ''
			for i in items:
				row = row + 1
				self.subs += str(row) +'\n'
				self.subs += seconds_to_hms(int(i[2])) + ' --> ' + seconds_to_hms(int(i[1])) + '\n'
				self.subs += i[0].decode('string_escape').replace('+',' ').replace('<br>','\n')
				self.subs += '\n\n' 
		try:
			self.subs = self.subs.encode("windows-1251")
		except:
			pass	
		return self.subs	
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<meta name="title" content="(.*?)"', self.source)
		if m:
			self.title = m.group(1)
		else:
			self.title = None
		return self.title
		

class vboxbg(WebVideoDownloader):
	re = '(www\.)?play.vbox.bg/([a-zA-Z0-9]+)/'
	website = 'http://vbox.bg'
		
	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
			
		m = re.search('videoID=(\d+)"', self.source)
		if m:
			params = urllib.urlencode({
				'id': m.group(1)
			})
			url = "http://vbox.bg/extras/player/play.php?%s" % params
			f = urllib.urlopen(url)
			self.returndata = f.read()
			m = re.search('url="(.*?)"', self.returndata)
			if m:
				videoUrl = m.group(1)
				videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
				self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title)+videoExt]]
			else:
				self.video_urls = []
			return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<meta name="description" content="(.*?)"', self.source)
		if m:
			self.title = m.group(1)
		else:
			self.title = None
		return self.title
	
class sportalbg(WebVideoDownloader):
	re = '(www\.)?sportal.bg/video\.php\?video=(\d+)'
	website = 'http://sportal.bg'
	headers = ['Cookie: adv_spam=1']
	
	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
					
		m = re.search('video_file_0=(.*?)&', self.source)

		if m:
			videoUrl = m.group(1)
			videoExt = '.flv'
			self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title)+videoExt]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		self.get_source()
		m = re.search(r'<td id="news_heading">(.*?)<\/td>', self.source, re.DOTALL)
		if m:
			self.title = urldecode(m.group(1)).strip()
		else:
			self.title = None
		
		return self.title

class gospodari(WebVideoDownloader):
	re = '(www\.)?gospodari.com/'
	website = 'http://gospodari.com'
	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
			
		m = re.search("{url: '(/videofiles/.*?)'", self.source)
		if m:
			videoUrl = 'http://www.gospodari.com'+m.group(1)
			videoExt = '.flv'
			self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<title>(.*?)\s*\|', self.source)
		if m:
			self.title = m.group(1)
		else:
			self.title = None
		
		return self.title

class newsbg(WebVideoDownloader):
	re = '(www\.)?news.ibox.bg/news/.*'
	website = 'http://news.ibox.bg'
		
	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
			
		m = re.search('http://ibox\.bg/load_flash_code\.php?.*?&flv=(.*?)&', self.source)

		if m:
			videoUrl = m.group(1)
			videoExt = '.flv'
			self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<title>(.*?)</title>', self.source)
		if m:
			self.title = m.group(1).decode("windows-1251")
		else:
			self.title = None
		
		return self.title

class mqstocom(WebVideoDownloader):
	re = '(www\.)?mqsto.com/video/(\d+)'
	website = 'http://mqsto.com'
		
	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search("so.addVariable\('file','(.*?)'\);", self.source)

		if m:
			videoUrl = 'http://mqsto.com/video/' + m.group(1)
			videoExt = '.flv'
			self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search("so.addVariable\('title','(.*?)'\);", self.source)
		if m:
			self.title = m.group(1)
		else:
			self.title = None
		
		return self.title

class mp3bg(WebVideoDownloader):
	re = '(www\.)?mp3.dnes.bg/video.php\?id=(\d+)'
	website = 'http://mp3.dnes.bg'
		
	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
			
		
		m = re.search('so.addVariable\("file", "(.*?)"\);', self.source)
		if m:
			m1 = re.search('(/playlist\.php.*)', m.group(1))
			if m1:
				html = self.http_request('http://mp3.dnes.bg'+m1.group(1))
				if html:
					m2 = re.search('<location>(.*?)</location>', html)
					if m2:
						videoUrl = m2.group(1)
						videoExt = '.flv'
						self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
			else:
				videoUrl = m.group(1)
				videoExt = '.flv'
				self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search(r'<span class="newsTitle">(.*?)</span>.*?<strong>(.*?)<\/strong>', self.source, re.DOTALL)
		if m:
			self.title = (m.group(1).decode('windows-1251') + ' - ' + m.group(2).decode('windows-1251'))
		else:
			self.title = None
		
		return self.title

class clipovete(WebVideoDownloader):
	re = '(www\.)?clipovete\.com/watch/(\d+)/(\d+)/'
	website = 'http://clipovete.com'
		
	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search('so\.addParam\("flashvars","&video=(.*?)&video_id=(\d+)"\);', self.source)
		if m:
			videoUrl = 'http://storage.puiako.com/clipovete.com/videos/%s.flv' % urldecode(m.group(1))
			videoExt = '.flv'
			self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<meta http-equiv="description" content="(.*?)"', self.source, re.DOTALL)
		if m:
			self.title = m.group(1)
		else:
			self.title = None
		
		return self.title
		
class clip4e(WebVideoDownloader):
	re = '(www\.)?clip4e\.com/play_(.*?)\.htm'
	website = 'http://clip4e.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m1 = re.search("\.addVariable\('cl_id',\s*'(.*?)'", self.source)
		if m1:
			html = self.http_request('http://www.clip4e.com/getflv.php?id=%s' % m1.group(1))
			m2 = re.search('<url>(.*?)<\/url>', html)
			if m2:
					videoUrl = m2.group(1)
					videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
					self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
			else:
				self.video_urls = []
				self.title = None
		else:
			self.video_urls = []
			self.title = None

		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<meta name="title" content="(.*?)"', self.source, re.DOTALL)
		if m:
			self.title = m.group(1).decode('windows-1251')
		else:
			self.title = None
		
		return self.title

	def get_subtitles(self):
		#TODO
		
		return
class databg(WebVideoDownloader):
	re = '(www\.)?video\.data\.bg\/(\d+)\/(\d+)'
	website = 'http://video.data.bg'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search('file=(http:\/\/vid.data.bg.*?)&', self.source)
		if m:
			url = m.group(1)
			self.video_urls = [[_('Video'), url, normalize_filename(self.title)+os.path.splitext(url)[1]]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<h1 class="viewpagettl".*?>(.*?)<\/h1>', self.source, re.DOTALL)
		if m:
			self.title = m.group(1)
		else:
			self.title = None
		
		return self.title

class screenbg(WebVideoDownloader):
	re = '(www\.)?screen\.bg\/.*?\/(\d+).html'
	website = 'http://screen.bg'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search("'FlashVars','file=(.*?)&amp;", self.source)
		if m:
			url = urldecode(m.group(1))
			self.video_urls = [[_('Video'), url, normalize_filename(self.title)+os.path.splitext(url)[1]]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<div class="videoTitle">(.*?)<\/div>', self.source, re.DOTALL)
		if m:
			self.title = m.group(1)
		else:
			self.title = None
		
		return self.title
		
class friendsbg(WebVideoDownloader):
	re = '(www\.)?friends\.bg\/video\/([a-z0-9]+)'
	website = 'http://friends.bg'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search(self.re, self.url)
		if m:
			url = 'http://www.friends.bg/files/video/flv/%s.flv' % m.group(2)
			self.video_urls = [[_('Video'), url, normalize_filename(self.title)+os.path.splitext(url)[1]]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<title>FRIENDS.bg\s*-\s*.*?:\s*(.*?)</title>', self.source, re.DOTALL)
		if m:
			self.title = m.group(1).decode('windows-1251')
		else:
			self.title = None
		
		return self.title
		
class crazybg(WebVideoDownloader):
	re = '(www\.)?crazy\.bg\/\?act=video&video_id=(\d+)'
	website = 'http://crazy.bg'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls

		m = re.search("nextMovie:\s*'(.*?)'", self.source)
		if m:
			url = urldecode(m.group(1))
			self.video_urls = [[_('Video'), url, normalize_filename(self.title)+os.path.splitext(url)[1]]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m1 = re.search(self.re, self.url)
		
		if m1:
			m2 = re.search('<a href="http:\/\/(www\.)?crazy\.bg\/\?act=video&video_id=%s.*?"\s+title="(.*?)"\s+class="path_link"' % m1.group(2), self.source, re.DOTALL)
			if m2:
				self.title = m2.group(2).decode('windows-1251')
			else:
				self.title = None
		else:
			self.title = None
		
		return self.title

class klip43ta(WebVideoDownloader):#JWPlayer
	re = '(www\.)?klip43ta.com/(.*?)/(.*?)-video_(.*?).html'
	website = 'http://klip43ta.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search("var\s+flashvars\s*=\s*?{.*?file:\s*'(.*?)',.*?type:\s*'video',", self.source, re.DOTALL)
		if m:
			url = m.group(1)
			self.video_urls = [[_('Video'), url, normalize_filename(self.title)+'.flv']]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<h2 class="h2_song">(.*?)</h2>.*?<h2 class="h2_artist">(.*?)</h2>', self.source, re.DOTALL)
		if m:
			self.title = m.group(2) + ' - ' + m.group(1)
		else:
			self.title = None
		
		return self.title
		
class youtube(WebVideoDownloader):
	re = ['(www\.)?youtube\.com', '(www\.)?youtu\.be']
	website = 'http://youtube.com'
	formats = { # based on http://en.wikipedia.org/wiki/YouTube#Quality_and_codecs and others...
		'5': ['flv', '240p FLV (H.263 / MP3 22050Hz) [FMT5]'],
		'6': ['flv', '360p FLV (H.263 / MP3 Mono) [FMT6]'],
		'34': ['flv', '360p MP4 (H.264 / AAC Stereo 44100Hz) [FMT34]'],
		'35': ['flv', '480p MP4 (H.264 / AAC Stereo 44100Hz) [FMT35]'],

		'18': ['mp4', '360p MP4 (H.264 / AAC Stereo 44100Hz) [FMT18]'],
		'22': ['mp4', '720p MP4 (H.264 / AAC Stereo 44100Hz) [FMT22]'],
		'37': ['mp4', '1080p MP4 (H.264 / AAC Stereo 44100Hz) [FMT37]'],
		'38': ['mp4', '3072p MP4 (H.264 / AAC Stereo 48000Hz) [FMT38]'],

		'13': ['3gp', '3GP [FMT13]'],
		'17': ['3gp', '144p 3GP (MPEG-4 Visual / AAC Stereo 44100Hz) [FMT17]'],

		'43': ['webm', '480p WebM (VP8 / OGG Stereo 44100Hz 96kb/s) [FMT43]'],
		'45': ['webm', '720p WebM (VP8 / OGG Stereo 44100Hz 128kb/s) [FMT45]'],
	}

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		self.video_urls = []
		m = re.search("'VIDEO_ID':\s*'(.*?)'", self.source)
		if m:
			videoID = m.group(1)
			m_fmt_url_map = re.search('fmt_url_map=(.*?)&',self.source)
			if m_fmt_url_map:
				fmt_url_map = urldecode(m_fmt_url_map.group(1))
				fmt_url_map = fmt_url_map.split(',')
				for fmt in fmt_url_map:
					fmt = fmt.split('|')
					format = fmt[0]
					url = fmt[1].replace('\/','/')
					if format in self.formats:
						ext = self.formats[format][0]
						description = self.formats[format][1]
					else:
						ext = 'flv'
						description = 'Video (%s)' % ext
					self.video_urls.append([description, url, normalize_filename(self.title) + '.' + ext])
			else:
				self.video_urls = []
			return self.video_urls
			
			
	def get_subtitles(self):
		if not self.video_urls:
			if not self.get_video_url():
				return None
		
		self.subs = None
		return self.subs

	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('id="eow-title".*?title="(.*?)"', self.source)
		if m:
			self.title = m.group(1)
		else:
			self.title = None
		return self.title

class yooyoo(WebVideoDownloader):#JWPlayer
	re = '(www\.)?yooyoo\.info\/(.*?)\/(.*?)-video_(.*?).html'
	website = 'http://yooyoo.info'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search("var\s+flashvars\s*=\s*?{.*?file:\s*'(.*?)',.*?type:\s*'video',", self.source, re.DOTALL)
		if m:
			url = m.group(1)
			self.video_urls = [[_('Video'), url, normalize_filename(self.title)+'.flv']]
		else:
			m = re.search("var\s+flashvars\s*=\s*?{.*?file:\s*'(http:\/\/(www\.)?youtube\.com\/watch\?v=.*?)'", self.source, re.DOTALL)
			if m:
				yt = youtube(m.group(1))
				if yt.detect():
					self.video_urls = yt.get_video_url()
				else:
					self.video_urls = []
			else:
				self.video_urls = []

		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<h2 class="h2_artist">(.*?)</h2>.*?<h2 class="h2_song">(.*?)</h2>', self.source, re.DOTALL)
		if m:
			self.title = m.group(1) + ' - ' + m.group(2)
		else:
			self.title = None
		
		return self.title

#TODO: NOT FUNCTIONAL		
class facebook(WebVideoDownloader): #TODO: login system
	re = '(www\.)?facebook\.com\/video\/video\.php\?v=(\d+)'
	website = 'http://facebook.com'
	login_url = 'https://login.facebook.com/login.php?login_attempt=1'
	login_data = {
		'email': ''
		, 'pass': ''
	}
	

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search('.addVariable("video_src",\s*"(.*?)")', self.source, re.DOTALL)
		if m:
			url = urldecode(m.group(1))
			self.video_urls = [[_('Video'), url, normalize_filename(self.title)+'.mp4']]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m1 = re.search('<h3 class="video_title.*?">(.*?)<\/h3>', self.source, re.DOTALL)
		m2 = re.search('<a href=".*?" class="video_owner_link">(.*?)</a>', self.source, re.DOTALL)
		if m1 and m2:
			self.title = m2.group(1) + ' - ' + m1.group(1)
		else:
			self.title = None
		
		return self.title

class bliptv(WebVideoDownloader):
	re = '(www\.)?blip\.tv\/file\/(\d+)'
	website = 'http://blip.tv'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		self.video_urls = []
		m = re.search('<select id="SelectFormat".*?>(.*?)<\/select>', self.source, re.DOTALL)
		if m:
			
			formats = re.findall('<option value="/file/(\d+)\?filename=(.*?)">(.*?)</option>',m.group(1))
			if formats:
				for format in formats:
					url = 'http://blip.tv/file/get/%s?referrer=blip.tv&source=1&use_direct=1&use_documents=1' % format[1]
					self.video_urls.append([unhtmlentities(format[2]), url, normalize_filename(self.title) + os.path.splitext(format[1])[1]])
			else:
				self.video_urls = []
			return self.video_urls
		else:
			self.video_urls = []
		return self.video_urls			
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<div id="EpisodeTitle">(.*?)<\/div>', self.source)
		if m:
			self.title = m.group(1)
		else:
			self.title = None
		return self.title

class dnesbg(WebVideoDownloader):
	re = '(www\.)?.dnes.bg\/.*'
	website = 'http://dnes.bg'
		
	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search('flashvars="file=(.*?)&', self.source)
		if m:
			m1 = re.search('(.*?/playlist\.php.*)', m.group(1))
			if m1:
				html = self.http_request(m1.group(1))
				if html:
					m2 = re.search('<location>(.*?)</location>', html)
					if m2:
						self.video_urls = [[_('Video'), m2.group(1), normalize_filename(self.title)+os.path.splitext(m2.group(1))[1]]]
			else:
				self.video_urls = [[_('Video'), m.group(1), normalize_filename(self.title)+os.path.splitext(m.group(1))[1]]]

		else:
			m1 = re.search('config=(http:\/\/video\.ibg\.bg\/playlistA\.php\?id=\d+)', self.source)
			if m1:
				playlistUrl = m1.group(1)
				xml = self.http_request(playlistUrl)
				m2 = re.search('"file":"(.*?)"', xml)
				if m2:	
					videoUrl = m2.group(1)
					videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
					self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
				else:
					self.video_urls = []
				
							
			else:
				self.video_urls = []
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search(r'<title>(.*?)</title>', self.source, re.DOTALL)
		if m:
			self.title = (m.group(1).decode('windows-1251'))
		else:
			self.title = None
		
		return self.title

class nnclips(WebVideoDownloader):#JWPlayer
	re = '(www\.)?99clips\.net\/musicvideo\.php\?vid=([0-9a-z]+)'
	website = 'http://99clips.net'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls

		m = re.search('flashvars="file=((http:\/\/(www\.)?youtube\.com\/watch\?v=.*?)&.*?)"', self.source, re.DOTALL)
		if m:
			if re.search('youtube\.com',m.group(2)):
				yt = youtube(m.group(2))
				if yt.detect():
					self.video_urls = yt.get_video_url()
				else:
					self.video_urls = []
			else:
				url = m.group(2)
				self.video_urls = [[_('Video'), url, normalize_filename(self.title)+'.flv']]
		else:
			self.video_urls = []

		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<h2 class="h2_song".*?>(.*?)</h2>.*?<h2 class="h2_artist".*?>(.*?)</h2>', self.source, re.DOTALL)
		if m:
			self.title = m.group(2) + ' - ' + m.group(1)
		else:
			self.title = None
		
		return self.title

class dailymotion(WebVideoDownloader):
	re = '(?:www\.)?dailymotion\.[a-z]{2,3}/video/([^_/]+)_([^/]+)'
	website = 'http://dailymotion.com'

		
	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search('&DMVIDEO=(.*?)&', self.source)
		if m:
			DMVIDEO = m.group(1)
			
		self.video_urls = []
		m = re.search(r'(?i)addVariable\(\"sequence\"\s*,\s*\"([^\"]*)\"\)', self.source)
		if m:
			parameters = urllib.unquote(m.group(1))
			m1 = re.search('"sdURL"\:"(.*?)"', parameters)
			if m1:
				videoUrl = m1.group(1).replace('\/','/')
				self.video_urls.append([_('Standard Definition'), videoUrl, normalize_filename(self.title)+os.path.splitext(videoUrl)[1]])

			m2 = re.search('"hqURL"\:"(.*?)"', parameters)
			if m2:
				videoUrl = m2.group(1).replace('\/','/')
				self.video_urls.append([_('Standard Definition HQ'), videoUrl, normalize_filename(self.title)+os.path.splitext(videoUrl)[1]])

			m3 = re.search('"hdURL"\:"(.*?)"', parameters)
			if m3:
				videoUrl = m3.group(1).replace('\/','/')
				self.video_urls.append([_('High Definition'), videoUrl, normalize_filename(self.title)+os.path.splitext(videoUrl)[1]])
		else:
			self.video_urls = None
			self.title = None
		return self.video_urls
		
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search(r'(?im)<title>Dailymotion\s*[\-:]\s*(.+?)</title>', self.source)
		if m:
			self.title = m.group(1)
		else:
			self.title = None
		
		return self.title
		
class jwak(WebVideoDownloader):#JWPlayer
	re = '(www\.)?jwak\.net\/media\/(\d*?)\/'
	website = 'http://jwak.net'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls

		m = re.search('.addVariable\("file","(.*?)"\)', self.source, re.DOTALL)
		if m:
			if re.search('youtube\.com',m.group(1)):
				yt = youtube(m.group(1))
				if yt.detect():
					self.video_urls = yt.get_video_url()
				else:
					self.video_urls = []
			else:
				videoUrl = m.group(1)
				self.video_urls = [[_('Video'), videoUrl, normalize_filename(self.title)+os.path.splitext(videoUrl)[1]]]
		else:
			self.video_urls = []

		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<title>Jwak\.Net -\s*(.*?)<\/title>', self.source, re.DOTALL)
		if m:
			self.title = m.group(1)
		else:
			self.title = None
		
		return self.title

class dtrailer(WebVideoDownloader):#JWPlayer
	re = '(www\.)?.*?\.?dtrailer.com/(.*?)/watch/(.*?)'
	website = 'http://dtrailer.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls

		m = re.search('.addVariable\("file","(.*?)"\)', self.source, re.DOTALL)
		
		if m:
			if re.search('youtube\.com',m.group(1)):
				yt = youtube(m.group(1))
				if yt.detect():
					self.video_urls = yt.get_video_url()
				else:
					self.video_urls = []
			else:
				videoUrl = 'http://www.dtrailer.com/dupload/trailerz/%s' % m.group(1)
				self.video_urls = [[_('Video'), videoUrl, normalize_filename(self.title)+os.path.splitext(videoUrl)[1]]]
		else:
			self.video_urls = []

		return self.video_urls

class dirbg_video(WebVideoDownloader):
	re = '(www\.)?video\.dir\.bg\/video_open\.php\?id=(\d+)'
	website = 'http://video.dir.bg'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m1 = re.search(self.re, self.url, re.DOTALL)
		if m1:
			videoID = m1.group(2)
			html = self.http_request('http://video.dir.bg/javascript/streaming.php?playlist=%s' % videoID)
			m2 = re.search('<streams><!\[CDATA\[(.*?)\]\]><\/streams>', html, re.DOTALL)
			if m2:
				videoUrl = m2.group(1)
				self.video_urls = [[_('Video'), videoUrl, normalize_filename(self.title)+os.path.splitext(urlparse(videoUrl).path)[1]]]
		else:
			self.video_urls = []

		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<title>(.*?)</title>', self.source, re.DOTALL)
		if m:
			self.title = m.group(1).decode("windows-1251")
		else:
			self.title = None
		
		return self.title

class soundbg(WebVideoDownloader):#JWPlayer
	re = '(www\.)?soundbg\.com\/(.*?)\/(.*?)-video_(.*?).html'
	website = 'http://soundbg.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search("var\s+flashvars\s*=\s*{.*?file:\s*'(.*?)',.*?type:\s*'video',", self.source, re.DOTALL)
		if m:
			videoUrl = m.group(1)
			videoExt = '.flv'
			self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
class movbg(WebVideoDownloader):#JWPlayer
	re = '(www\.)?mov\.bg\/video\/(.*?)/'
	website = 'http://mov.bg'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search("player\.addVariable\('file',\s*'(.*?)'\);", self.source, re.DOTALL)
		if m:
			videoUrl = m.group(1)
			self.video_urls = [[_('Video'), videoUrl, normalize_filename(self.title)+'.flv']]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<h1 class="title.*?>(.*?)<\/h1>', self.source, re.DOTALL)
		if m:
			self.title = m.group(1).replace("\n"," ")
		else:
			self.title = None
		
		return self.title

class megavideo(WebVideoDownloader):
	re = '(www\.)?megavideo\.com\/\?v=(.*?)'
	website = 'http://megavideo.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m1 = re.compile('<param name="movie" value="http://www.megavideo.com/v/(.+?)"></param>').findall(self.source)
		if m1:
			videoID = m1[0][:8]
			headers = [
				'Referer: http://www.megavideo.com/'
			]
			xml = self.http_request('http://www.megavideo.com/xml/videolink.php?v='+videoID, None, headers)
			s = re.compile(' s="(.+?)"').findall(xml)
			k1 = re.compile(' k1="(.+?)"').findall(xml)
			k2 = re.compile(' k2="(.+?)"').findall(xml)
			un = re.compile(' un="(.+?)"').findall(xml)
			if s and k1 and k2 and un:
				videoUrl = "http://www" + s[0] + ".megavideo.com/files/" + self.decrypt(un[0], k1[0], k2[0]) + "/"+'?.flv'
				self.video_urls = [[_('Video (%s)') % 'flv', videoUrl, normalize_filename(self.title)+'.flv']]
			else:
				self.video_urls = []
		else:
			self.video_urls = []
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('flashvars.title\s*=\s*"(.*?)";', self.source, re.DOTALL)
		if m:
			self.title = urldecode(m.group(1)).replace('+',' ')
		else:
			self.title = None
		
		return self.title
		
	def ajoin(self, arr):
		strtest = ''
		for num in range(len(arr)):
			strtest = strtest + str(arr[num])
		return strtest
	
	def asplit(self, mystring):
		arr = []
		for num in range(len(mystring)):
			arr.append(mystring[num])
		return arr
	
	def decrypt(self, str1, key1, key2):
		__reg1 = []
		__reg3 = 0
		while (__reg3 < len(str1)):
			__reg0 = str1[__reg3]
			holder = __reg0
			
			
			if (holder == "0"):
				__reg1.append("0000")
			else:
				__reg1.append(str(bin(eval('0x'+__reg0))).replace('0b','').rjust(4,'0'))
			
			__reg3 = __reg3 + 1

		mtstr = self.ajoin(__reg1)
		__reg1 = self.asplit(mtstr)
		__reg6 = []
		__reg3 = 0
		while (__reg3 < 384):
	
			key1 = (int(key1) * 11 + 77213) % 81371
			key2 = (int(key2) * 17 + 92717) % 192811
			__reg6.append((int(key1) + int(key2)) % 128)
			__reg3 = __reg3 + 1
	
		__reg3 = 256
		while (__reg3 >= 0):

			__reg5 = __reg6[__reg3]
			__reg4 = __reg3 % 128
			__reg8 = __reg1[__reg5]
			__reg1[__reg5] = __reg1[__reg4]
			__reg1[__reg4] = __reg8
			__reg3 = __reg3 - 1
		
		__reg3 = 0
		while (__reg3 < 128):
	
			__reg1[__reg3] = int(__reg1[__reg3]) ^ int(__reg6[__reg3 + 256]) & 1
			__reg3 = __reg3 + 1

		__reg12 = self.ajoin(__reg1)
		__reg7 = []
		__reg3 = 0
		while (__reg3 < len(__reg12)):
			__reg9 = __reg12[__reg3:__reg3 + 4]
			__reg7.append(__reg9)
			__reg3 = __reg3 + 4
		
		__reg2 = []
		__reg3 = 0
		while (__reg3 < len(__reg7)):
			__reg0 = __reg7[__reg3]
			holder2 = __reg0
			
			if (holder2 == "0000"):
				__reg2.append("0")
			else:
				__reg2.append(hex(int(__reg0,2))[-1])
											
			__reg3 = __reg3 + 1

		endstr = self.ajoin(__reg2)
		return endstr	

class metacafe(WebVideoDownloader):#JWPlayer
	re = '(?:http://)?(?:www\.)?metacafe\.com/watch/([^/]+)/([^/]+)/.*'
	website = 'http://metacafe.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m1 = re.match(self.re, self.url)
		
		if m1:
			videoID = m1.group(1) 
			m2 = re.match(r'^yt-(.*)$', videoID)
			if m2:
				yt = youtube('http://www.youtube.com/watch?v=%s' % m.group(1))
				if yt.detect():
					self.video_urls = yt.get_video_url()
				else:
					self.video_urls = []
			else:
				html = self.http_request('http://www.metacafe.com/watch/%s/' % videoID)
				mobj = re.search(r'(?m)&mediaURL=([^&]+)', html)
				if mobj is not None:
					mediaURL = urllib.unquote(mobj.group(1))
					videoExt = mediaURL[-3:]
					mobj = re.search(r'(?m)&gdaKey=(.*?)&', html)
					if mobj is None:
						videoUrl = mediaURL
					else:
						gdaKey = mobj.group(1)
						videoUrl = '%s?__gda__=%s' % (mediaURL, gdaKey)
				else:
					mobj = re.search(r' name="flashvars" value="(.*?)"', html)
					if mobj is None:
						self._downloader.trouble(u'ERROR: unable to extract media URL')
						return
					vardict = parse_qs(mobj.group(1))
					if 'mediaData' not in vardict:
						self._downloader.trouble(u'ERROR: unable to extract media URL')
						return
					mobj = re.search(r'"mediaURL":"(http.*?)","key":"(.*?)"', vardict['mediaData'][0])
					if mobj is None:
						self._downloader.trouble(u'ERROR: unable to extract media URL')
						return
					mediaURL = mobj.group(1).replace('\\/', '/')
					videoExt = mediaURL[-3:]
					videoUrl = '%s?__gda__=%s' % (mediaURL, mobj.group(2))		
				
				self.video_urls = [[_('Video (%s)') % videoExt, videoUrl, normalize_filename(self.title)+'.'+videoExt]]		
		else:
			self.video_urls = []

		return self.video_urls
	
class googlevideo(WebVideoDownloader):
	re = r'(?:http://)?video\.google\.(?:com(?:\.au)?|co\.(?:uk|jp|kr|cr)|ca|de|es|fr|it|nl|pl)/videoplay\?docid=([^\&]+).*'
	website = 'http://video.google.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		html = self.http_request('http://video.google.com/videoplay?docid=%s&hl=en&oe=utf-8' % self.detected.group(1))
		
		videoExt = 'mp4'
		
		m1 = re.search(r"download_url:'([^']+)'", html)
		if m1:
			videoExt = 'mp4'
			videoUrl = urllib.unquote(mobj.group(1))
			videoUrl = videoUrl.replace('\\x3d', '\x3d')
			videoUrl = videoUrl.replace('\\x26', '\x26')
		
			self.video_urls = [[_('Video'), videoUrl, normalize_filename(self.title) + '.' + videoExt]]
		else:
			m2 = re.search(r"(?i)videoUrl\\x3d(.+?)\\x26", html)
			if m2:
				videoExt = 'flv'
				videoUrl = urllib.unquote(m2.group(1))
				videoUrl = videoUrl.replace('\\x3d', '\x3d')
				videoUrl = videoUrl.replace('\\x26', '\x26')
				self.video_urls = [[_('Video (%s)') % videoExt, videoUrl, normalize_filename(self.title) + '.' + videoExt]]
			else:
				self.video_urls = []
				self.title = None

		return self.video_urls

class soundcloud(WebVideoDownloader):
	re = '(www\.)?soundcloud\.com\/(.*?)\/([a-zA-Z/]+)'
	website = 'http://soundcloud.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search('<a href="(.*?/download)" class="download pl-button"', self.source)
		if m:
			audioUrl = 'http://soundcloud.com' + m.group(1)
			self.video_urls = [[_('MP3'), audioUrl, normalize_filename(self.title)+'.mp3']]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<meta content="(.*?)" property="og:title"', self.source)
		if m:
			self.title = m.group(1)
		else:
			self.title = None
		
		return self.title

class photobucket(WebVideoDownloader):
	re = r'(?:http://)?(?:[a-z0-9]+\.)?photobucket\.com/.*'
	website = 'http://photobucket.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search('<link rel="video_src" href=".*\?file=([^"]+)" />', self.source)
		if m:
			videoUrl = urllib.unquote(m.group(1))
			videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
			self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title)+videoExt]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<span class="title">(.*?)<\/span>', self.source)
		if m:
			self.title = m.group(1).decode('utf-8')
		else:
			self.title = None
		
		return self.title

class jupiterbroadcasting(WebVideoDownloader):
	re = '(www\.)?jupiterbroadcasting\.com\/\?p=(\d+)'
	website = 'http://jupiterbroadcasting.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		self.video_urls = []
		m = re.findall("podPressShowHidePlayer\('\d+', '(.*?)', \d+, \d+, '.*?', '.*?', '(.*?)', '.*?'\);", self.source, re.DOTALL)
		if m:
			for row in m:
				videoUrl = row[0]
				videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
				self.video_urls.append([row[1], videoUrl, normalize_filename(self.title) + videoExt])
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls

class boozho(WebVideoDownloader):
	re = '(www\.)?boozho\.com\/view\.php\?v=([a-zA-Z0-9]+)'
	website = 'http://boozho.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		html = self.http_request('http://www.boozho.com/player_playlist.php?v=%s' % self.detected.group(2))
		if html:
			m = re.search('<movie_path>(.*?)<\/movie_path>', html)
			if m:
				videoUrl = 'http://www.boozho.com/%s' % m.group(1)
				videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
				self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title)+videoExt]]
			else:
				self.video_urls = []
		else:
			self.video_urls = []
		return self.video_urls

class fivemin(WebVideoDownloader):
	re = r'(www\.)?5min.com/Video/(.*)-(\d+)'
	website = 'http://5min.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search('videoUrl=(.*?)&', self.source)
		if m:
			videoUrl = urllib.unquote(m.group(1))
			videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
			self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title)+videoExt]]
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<h1 class="videoTitle">(.*?)<\/h1>', self.source)
		if m:
			self.title = m.group(1).decode('utf-8')
		else:
			self.title = None
		
		return self.title

class o23video(WebVideoDownloader):
	re = r'(www\.)?123video.nl\/playvideos\.asp\?MovieID=(\d+)'
	website = 'http://123video.nl'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m1 = re.search('mediaSrc=(.*?)&', self.source)
		if m1:
			videoID = m1.group(1)
			headers = [
				'Referer: %s' % self.url
				, 'Connection: keep-alive'
			]
			xml = self.http_request('http://www.123video.nl/initialize_player_v3.asp','<movie><id>%s</id></movie>' % videoID, headers, True)
			m2 = re.search('MediaIP="(.*?)"', xml)
			if m2:
				MediaIP = m2.group(1)
				videoUrl = 'http://%s/%s/%s.flv' % (MediaIP, videoID[:3], videoID)
				videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
				self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title)+videoExt]]
			else:
				self.video_urls = []
		else:
			self.video_urls = []
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('\?title=(.*?)&amp;', self.source)
		if m:
			self.title = m.group(1).decode('utf-8')
		else:
			self.title = None
		
		return self.title

class aniboom(WebVideoDownloader):
	re = '(www\.)?aniboom.com/animation-video/(\d+)(/(.*?)/)?'
	website = 'http://aniboom.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m1 = re.search('animationId=(.*?)"', self.source)
		if m1:
			videoID = m1.group(1)

			html = self.http_request('http://www.aniboom.com/animations/player/handlers/animationDetails.aspx?mode=&movieid=%s' % videoID)
			m2 = re.search('flv=(.*?)&', html)
			if m2:
				videoUrl = urllib.unquote(m2.group(1))
				videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
				self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title)+videoExt]]
			else:
				self.video_urls = []
		else:
			self.video_urls = []
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<meta name="title" content="(.*?)"', self.source)
		if m:
			self.title = m.group(1).decode('utf-8')
		else:
			self.title = None
		
		return self.title
		
class atom(WebVideoDownloader):
	re = '(www\.)?atom\.com/(.*)/(.*)/'
	website = 'http://atom.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		self.video_urls = []
		m1 = re.search("contentUri: '(.*?)'", self.source)
		if m1:
			uri = m1.group(1)
			xml1 = self.http_request('http://www.atom.com/atom/feed/rss/?uri=%s' % uri)
			m2 = re.search('<media:content.*?url="(.*?)"/>', xml1)
			if m2:
				headers = ['Referer: %s' % self.url]
				xml2 = self.http_request(m2.group(1), None, headers)
				m3 = re.findall('<rendition.*?bitrate="(.*?)" width="(.*?)" height="(.*?)" type="(.*?)">.*?<src><!\[CDATA\[(.*?)\]\]><\/src>', xml2, re.DOTALL)
				if m3:
					for row in m3:
						videoUrl = row[4]
						videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
						desc = '%s %sx%s@%s kbps' % (row[3],row[1],row[2],row[0])
						self.video_urls.append([_('Video (%s)') % desc, videoUrl, normalize_filename(self.title)+videoExt])
				else:
					self.video_urls = []
			else:
				self.video_urls = []
		else:
			self.video_urls = []
		return self.video_urls
	
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<meta name="title" content="(.*?)"', self.source)
		if m:
			self.title = m.group(1).decode('utf-8')
		else:
			self.title = None
		
		return self.title

class bebo(WebVideoDownloader):
	re = '(www\.)?bebo\.com'
	website = 'http://bebo.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m = re.search('&quot;http://www.youtube.com/v/([0-9A-Za-z_-]+)&quot;', self.source, re.DOTALL)
		if m:
				yt = youtube('http://www.youtube.com/watch?v=%s' % m.group(1))
				if yt.detect():
					self.video_urls = yt.get_video_url()
				else:
					self.video_urls = []
		else:
				self.video_urls = []

		return self.video_urls

class demoscene(WebVideoDownloader):
	re = '(www\.)?demoscene\.tv'
	website = 'http://demoscene.tv'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		self.video_urls = []
		m = re.findall("showFlashPlayer\('(.*?)','(\d+)','(\d+)',(\d+),(\d+)", self.source)
		if m:
			for row in m:
				videoUrl = urllib.unquote(row[0])
				videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
				videoTitle = '%s %sx%s' % (videoExt[1:], row[3], row[4])
				self.video_urls.append([_('Video (%s)') % videoTitle, videoUrl, normalize_filename(self.title) + videoExt])
		else:
			self.video_urls = []
			self.title = None
		return self.video_urls
		
class clipjunkie(WebVideoDownloader):
	re = '(www\.)?clipjunkie\.com\/(.*?)\.html'
	website = 'http://clipjunkie.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
				
		if self.detected:
			videoUrl = 'http://videos.clipjunkie.com/videos/%s.flv' % self.detected.group(2)
			videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
			self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
		else:
			self.video_urls = []
		return self.video_urls		

class collegehumor(WebVideoDownloader):
	re = '(www\.)?collegehumor\.com\/video:(\d+)'
	website = 'http://collegehumor.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		if self.detected:
			videoID = self.detected.group(2)
			xml = self.http_request('http://www.collegehumor.com/moogaloop/video:%s' % videoID)
			m = re.search('<file>(.*?)<\/file>', xml)
			if m:
				videoUrl = m.group(1)
				videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
				self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
			else:
				self.video_urls = []
		else:
			self.video_urls = []

		return self.video_urls

class ustream(WebVideoDownloader):
	re = '(www\.)?ustream\.tv\/'
	website = 'http://ustream.tv'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m1 = re.search('ustream.vars.liveHttpUrl="(.*?)"', self.source)
		if m1:
			m3u = self.http_request(m1.group(1).replace('\/','/'))
			videoUrl = m1.group(1).replace('\/','/')
			videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
			self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
		else:
			m2 = re.search('videopic\/"(.*?)"_320x240', self.source)
			if m2:
				videoUrl = 'http://ustream.vo.llnwd.net/pd5/%s.flv' % m2.group(1)
				videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
				self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
			else:
				self.video_urls = []
		return self.video_urls


class liveleak(WebVideoDownloader):
	re = '(www\.)?liveleak.com/.*'
	website = 'http://liveleak.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m1 = re.search('file_embed_tag%3D(.*?)%26token', self.source)
		if m1:
			xml = self.http_request('http://www.liveleak.com/playlist.php?file_embed_tag=%s' % m1.group(1))
			m2 = re.search('<location>(.*?)<\/location>', xml)
			if m2:
				videoUrl = m2.group(1)
				videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
				self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
			else:
				self.video_urls = []
		else:
			self.video_urls = []
		return self.video_urls

#TODO: NOT FUNCTIONAL
class yahoovideo(WebVideoDownloader):
	re = '(www\.)?video.yahoo.com/.*'
	website = 'http://video.yahoo.com'
	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		m1 = re.search('so.addVariable\("id",\s*"(.*?)"', self.source)
		if m1:
			xml = self.http_request('http://cosmos.bcst.yahoo.com/up/yep/process/getPlaylistFOP.php?node_id=%s&tech=flash&mode=playlist&bitrate=300&null&rd=video.yahoo.com&tk=null' % m1.group(1))
			m2 = re.search('<STREAM APP="(.*?)" FULLPATH="(.*?)"', xml)
			if m2:
				videoUrl = urldecode(m2.group(1) + '/' + m2.group(2))
				m3 = re.search('m=(.*?)&', videoUrl)
				if m3:
					videoExt = mimetypes.guess_extension(m3.group(1))
				else:
					videoExt = '.flv'
				self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]
			else:
				self.video_urls = []
		else:
			self.video_urls = []
		return self.video_urls

class gametrailers(WebVideoDownloader):
	re = '(www\.)?gametrailers\.com'
	website = 'http://gametrailers.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		self.video_urls = []
		m1 = re.search('var config_url\s*=\s*"(.*?)";', self.source)
		if m1:
			url = 'http://www.gametrailers.com' + urldecode(m1.group(1))
			xml = self.http_request(url)
			m2 = re.search('<player>.*?<feed.*?>(.*?)<\/feed>', xml, re.DOTALL)
			if m2:
				url2 = urldecode(m2.group(1).replace('&amp;','&').strip())
				xml2 = self.http_request(url2)
				m3 = re.search('<media:content.*?medium="video".*?url="(.*?)"', xml2, re.DOTALL)
				if m3:
					xml3 = self.http_request(m3.group(1).replace('&amp;','&'))

					m4 = re.findall('<rendition bitrate="(.*?)".*?type="(.*?)">.*?<src>(.*?)</src>', xml3, re.DOTALL)
					if m4:
						for row in m4:
							videoUrl = row[2].strip()
							videoExt = mimetypes.guess_extension(row[1])
							self.video_urls.append([_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt])
		return self.video_urls

	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<div class="movieTitle"><a>(.*?)<\/a><\/div>', self.source)
		if m:
			self.title = m.group(1).decode('utf-8')
		else:
			self.title = None
		
		return self.title
		
class gamespot(WebVideoDownloader):
	re = '(www\.)?gamespot\.com\/'
	website = 'http://gamespot.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		self.video_urls = []
		m1 = re.search('paramsXML=(.*?)&quot;', self.source)
		if m1:
			xml = self.http_request(urldecode(m1.group(1)))
			m2 = re.findall('<videoFile>.*?<filePath>(.*?)</filePath>.*?<rate>(.*?)</rate>', xml, re.DOTALL)
			if m2:
				for row in m2:
					videoUrl = row[0]
					videoExt = os.path.splitext(urlparse(videoUrl).path)[1]
					info = '%s @ %skb/s' % (videoExt[1:],row[1])
					self.video_urls.append([_('Video (%s)') % info, videoUrl, normalize_filename(self.title) + videoExt])

		return self.video_urls
		

	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<h2 class="video_title">(.*?)<\/h2>', self.source, re.DOTALL)
		if m:
			self.title = m.group(1).decode('utf-8')
		else:
			self.title = None
		
		return self.title
		
class vimeo(WebVideoDownloader):
	re = '(www\.)?vimeo\.com\/(hd#)?(\d+)'
	website = 'http://vimeo.com'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls
		
		self.video_urls = []
		channel_id = ''
		m1 = re.search('id="channel_id" value="(\d+)"', self.source)
		if m1:
			channel_id = m1.group(1)
			
		if channel_id:
			channel_params = 'context=channel&context_id=%s' % channel_id
		else:
			channel_params = ''
		
		videoID = self.detected.group(3)
		hd = self.detected.group(2)
		if hd:
			videoExt = '.mp4'
		else:
			videoExt = '.flv'
			
		xml = self.http_request('http://vimeo.com/moogaloop/load/clip:%s/local?%s' % (videoID, channel_params))
		m2 = re.search('<request_signature>(.*?)<\/request_signature>', xml, re.DOTALL)
		if m2:
			signature = m2.group(1)
			m3 = re.search('<request_signature_expires>(.*?)<\/request_signature_expires>', xml, re.DOTALL)
			if m2:
				signature_expires = m3.group(1)
				videoUrl = 'http://vimeo.com/moogaloop/play/clip:%s/%s/%s/?q=sd' % (videoID, signature, signature_expires)
				self.video_urls.append([_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt])
		return self.video_urls
		

	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		videoID = self.detected.group(3)
		channel_id = ''
		m1 = re.search('id="channel_id" value="(\d+)"', self.source)
		if m1:
			channel_id = m1.group(1)
			
		if channel_id:
			channel_params = 'context=channel&context_id=%s' % channel_id
		else:
			channel_params = ''
		xml = self.http_request('http://vimeo.com/moogaloop/load/clip:%s/local?%s' % (videoID, channel_params))
		m2 = re.search('<caption>(.*?)<\/caption>', xml, re.DOTALL)
		if m2:
			self.title = m2.group(1).decode('utf-8')
		else:
			self.title = None
		
		return self.title		

class btv(WebVideoDownloader):
	re = r'(www\.)?btv.bg/'
	website = 'http://btv.bg'

	def get_video_url(self):
		if not self.source:
			if not self.get_source():
				self.video_urls = []
				return self.video_urls
		if not self.title:
			if not self.get_title():
				self.video_urls = []
				return self.video_urls

		self.video_urls = []
		m = re.search("showPlayer\(.*?,'(\d+\.\d+\.\d+\.\d+(\:\d+)?)','(.*?)',(\d+),'http://.*?','.*?', '.*?'\);", self.source)

		if m:
			playpath = m.group(3)
			swfUrl = 'http://images.btv.bg/fplayer/flowplayer.commercial-3.2.5.swf'
			videoUrl = 'rtmp://%s/vodedge\t<playpath>%s\t<swfUrl>%s\t<pageUrl>%s' % ('94.155.44.74:80', playpath, swfUrl, self.url)
			videoExt = '.flv'
			self.video_urls = [[_('Video (%s)') % videoExt[1:], videoUrl, normalize_filename(self.title) + videoExt]]

		return self.video_urls
		
	def get_title(self):
		if not self.source:
			if not self.get_source():
				self.title = None
				return None
		m = re.search('<title>(.*?)</title>', self.source)
		if m:
			self.title = m.group(1).decode("windows-1251")
		else:
			self.title = None
		
		return self.title
		
available = [
	vbox7
	, vboxbg
	, sportalbg
	, gospodari
	, newsbg
	, mqstocom
	, mp3bg
	, clipovete
	, clip4e
	, databg
	, screenbg
	, friendsbg
	, crazybg
	, klip43ta
	, yooyoo
	, bliptv
	, dnesbg
	, youtube
	, nnclips
	, dailymotion
	, jwak
	, dtrailer
	, dirbg_video
	, soundbg
	, movbg
	, megavideo
	, metacafe
	, googlevideo
	, soundcloud
	, photobucket
	, jupiterbroadcasting
	, boozho
	, fivemin
	, o23video
	, aniboom
	, atom
	, bebo
	, demoscene
	, clipjunkie
	, collegehumor
	, ustream
	, liveleak
	#, yahoovideo
	, gametrailers
	, gamespot
	, vimeo
	, btv
]

if __name__ == '__main__':
	if len(sys.argv) > 1:
		downloader = None
		for plugin in available:
			d = plugin(sys.argv[1])
			if d.detect():
				downloader = d
				break
		if downloader:
			title = downloader.get_title().replace('\r','').replace('\n',' ')
			url_list = downloader.get_video_url()
			print title
			for row in url_list:
				print row[0] + '	' + row[1]
		else:
			print _('Invalid URL!')
			sys.exit(1)
	else:
		print _('Usage: %s [URL]') % sys.argv[0]
		listing = []
		for plugin in available:
			listing.append(plugin.website)
		listing.sort()
		print ''
		print _('viDown supports %(sitecount)d sites: %(sitelist)s') % ({'sitecount': len(listing), 'sitelist': '\n'+'\n'.join(listing)})
		
